Class: Google::Cloud::Spanner::Admin::Database::V1::DatabaseAdminClient
- Inherits:
-
Object
- Object
- Google::Cloud::Spanner::Admin::Database::V1::DatabaseAdminClient
- Defined in:
- lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb
Overview
Cloud Spanner Database Admin API
The Cloud Spanner Database Admin API can be used to create, drop, and list databases. It also enables updating the schema of pre-existing databases.
Constant Summary collapse
- SERVICE_ADDRESS =
The default address of the service.
"spanner.googleapis.com".freeze
- DEFAULT_SERVICE_PORT =
The default port of the service.
443- GRPC_INTERCEPTORS =
The default set of gRPC interceptors.
[]
- DEFAULT_TIMEOUT =
30- ALL_SCOPES =
The scopes needed to make gRPC calls to all of the methods defined in this service.
[ "https://www.googleapis.com/auth/cloud-platform", "https://www.googleapis.com/auth/spanner.admin" ].freeze
Class Method Summary collapse
-
.database_path(project, instance, database) ⇒ String
Returns a fully-qualified database resource name string.
-
.instance_path(project, instance) ⇒ String
Returns a fully-qualified instance resource name string.
Instance Method Summary collapse
-
#create_database(parent, create_statement, extra_statements: nil, options: nil) ⇒ Google::Gax::Operation
Creates a new Cloud Spanner database and starts to prepare it for serving.
-
#drop_database(database, options: nil) {|result, operation| ... } ⇒ Object
Drops (aka deletes) a Cloud Spanner database.
-
#get_database(name, options: nil) {|result, operation| ... } ⇒ Google::Spanner::Admin::Database::V1::Database
Gets the state of a Cloud Spanner database.
-
#get_database_ddl(database, options: nil) {|result, operation| ... } ⇒ Google::Spanner::Admin::Database::V1::GetDatabaseDdlResponse
Returns the schema of a Cloud Spanner database as a list of formatted DDL statements.
-
#get_iam_policy(resource, options: nil) {|result, operation| ... } ⇒ Google::Iam::V1::Policy
Gets the access control policy for a database resource.
-
#initialize(credentials: nil, scopes: ALL_SCOPES, client_config: {}, timeout: DEFAULT_TIMEOUT, metadata: nil, exception_transformer: nil, lib_name: nil, lib_version: "") ⇒ DatabaseAdminClient
constructor
A new instance of DatabaseAdminClient.
-
#list_databases(parent, page_size: nil, options: nil) {|result, operation| ... } ⇒ Google::Gax::PagedEnumerable<Google::Spanner::Admin::Database::V1::Database>
Lists Cloud Spanner databases.
-
#set_iam_policy(resource, policy, options: nil) {|result, operation| ... } ⇒ Google::Iam::V1::Policy
Sets the access control policy on a database resource.
-
#test_iam_permissions(resource, permissions, options: nil) {|result, operation| ... } ⇒ Google::Iam::V1::TestIamPermissionsResponse
Returns permissions that the caller has on the specified database resource.
-
#update_database_ddl(database, statements, operation_id: nil, options: nil) ⇒ Google::Gax::Operation
Updates the schema of a Cloud Spanner database by creating/altering/dropping tables, columns, indexes, etc.
Constructor Details
#initialize(credentials: nil, scopes: ALL_SCOPES, client_config: {}, timeout: DEFAULT_TIMEOUT, metadata: nil, exception_transformer: nil, lib_name: nil, lib_version: "") ⇒ DatabaseAdminClient
Returns a new instance of DatabaseAdminClient.
150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256 257 258 259 260 261 262 263 264 265 266 267 268 269 270 271 272 273 274 275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 295 296 297 298 299 300 301 302 303 304 305 |
# File 'lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb', line 150 def initialize \ credentials: nil, scopes: ALL_SCOPES, client_config: {}, timeout: DEFAULT_TIMEOUT, metadata: nil, exception_transformer: nil, lib_name: nil, lib_version: "" # These require statements are intentionally placed here to initialize # the gRPC module only when it's required. # See https://github.com/googleapis/toolkit/issues/446 require "google/gax/grpc" require "google/spanner/admin/database/v1/spanner_database_admin_services_pb" credentials ||= Google::Cloud::Spanner::Admin::Database::V1::Credentials.default @operations_client = OperationsClient.new( credentials: credentials, scopes: scopes, client_config: client_config, timeout: timeout, lib_name: lib_name, lib_version: lib_version, ) if credentials.is_a?(String) || credentials.is_a?(Hash) updater_proc = Google::Cloud::Spanner::Admin::Database::V1::Credentials.new(credentials).updater_proc end if credentials.is_a?(GRPC::Core::Channel) channel = credentials end if credentials.is_a?(GRPC::Core::ChannelCredentials) chan_creds = credentials end if credentials.is_a?(Proc) updater_proc = credentials end if credentials.is_a?(Google::Auth::Credentials) updater_proc = credentials.updater_proc end package_version = Google::Cloud::Spanner::VERSION google_api_client = "gl-ruby/#{RUBY_VERSION}" google_api_client << " #{lib_name}/#{lib_version}" if lib_name google_api_client << " gapic/#{package_version} gax/#{Google::Gax::VERSION}" google_api_client << " grpc/#{GRPC::VERSION}" google_api_client.freeze headers = { :"x-goog-api-client" => google_api_client } headers.merge!() unless .nil? client_config_file = Pathname.new(__dir__).join( "database_admin_client_config.json" ) defaults = client_config_file.open do |f| Google::Gax.construct_settings( "google.spanner.admin.database.v1.DatabaseAdmin", JSON.parse(f.read), client_config, Google::Gax::Grpc::STATUS_CODE_NAMES, timeout, page_descriptors: PAGE_DESCRIPTORS, errors: Google::Gax::Grpc::API_ERRORS, metadata: headers ) end # Allow overriding the service path/port in subclasses. service_path = self.class::SERVICE_ADDRESS port = self.class::DEFAULT_SERVICE_PORT interceptors = self.class::GRPC_INTERCEPTORS @database_admin_stub = Google::Gax::Grpc.create_stub( service_path, port, chan_creds: chan_creds, channel: channel, updater_proc: updater_proc, scopes: scopes, interceptors: interceptors, &Google::Spanner::Admin::Database::V1::DatabaseAdmin::Stub.method(:new) ) @list_databases = Google::Gax.create_api_call( @database_admin_stub.method(:list_databases), defaults["list_databases"], exception_transformer: exception_transformer, params_extractor: proc do |request| {'parent' => request.parent} end ) @create_database = Google::Gax.create_api_call( @database_admin_stub.method(:create_database), defaults["create_database"], exception_transformer: exception_transformer, params_extractor: proc do |request| {'parent' => request.parent} end ) @get_database = Google::Gax.create_api_call( @database_admin_stub.method(:get_database), defaults["get_database"], exception_transformer: exception_transformer, params_extractor: proc do |request| {'name' => request.name} end ) @update_database_ddl = Google::Gax.create_api_call( @database_admin_stub.method(:update_database_ddl), defaults["update_database_ddl"], exception_transformer: exception_transformer, params_extractor: proc do |request| {'database' => request.database} end ) @drop_database = Google::Gax.create_api_call( @database_admin_stub.method(:drop_database), defaults["drop_database"], exception_transformer: exception_transformer, params_extractor: proc do |request| {'database' => request.database} end ) @get_database_ddl = Google::Gax.create_api_call( @database_admin_stub.method(:get_database_ddl), defaults["get_database_ddl"], exception_transformer: exception_transformer, params_extractor: proc do |request| {'database' => request.database} end ) @set_iam_policy = Google::Gax.create_api_call( @database_admin_stub.method(:set_iam_policy), defaults["set_iam_policy"], exception_transformer: exception_transformer, params_extractor: proc do |request| {'resource' => request.resource} end ) @get_iam_policy = Google::Gax.create_api_call( @database_admin_stub.method(:get_iam_policy), defaults["get_iam_policy"], exception_transformer: exception_transformer, params_extractor: proc do |request| {'resource' => request.resource} end ) @test_iam_permissions = Google::Gax.create_api_call( @database_admin_stub.method(:test_iam_permissions), defaults["test_iam_permissions"], exception_transformer: exception_transformer, params_extractor: proc do |request| {'resource' => request.resource} end ) end |
Class Method Details
.database_path(project, instance, database) ⇒ String
Returns a fully-qualified database resource name string.
102 103 104 105 106 107 108 |
# File 'lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb', line 102 def self.database_path project, instance, database DATABASE_PATH_TEMPLATE.render( :"project" => project, :"instance" => instance, :"database" => database ) end |
.instance_path(project, instance) ⇒ String
Returns a fully-qualified instance resource name string.
114 115 116 117 118 119 |
# File 'lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb', line 114 def self.instance_path project, instance INSTANCE_PATH_TEMPLATE.render( :"project" => project, :"instance" => instance ) end |
Instance Method Details
#create_database(parent, create_statement, extra_statements: nil, options: nil) ⇒ Google::Gax::Operation
Creates a new Cloud Spanner database and starts to prepare it for serving.
The returned long-running operation will
have a name of the format <database_name>/operations/<operation_id> and
can be used to track preparation of the database. The
metadata field type is
CreateDatabaseMetadata.
The response field type is
Database, if successful.
428 429 430 431 432 433 434 435 436 437 438 439 440 441 442 443 444 445 446 447 448 |
# File 'lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb', line 428 def create_database \ parent, create_statement, extra_statements: nil, options: nil req = { parent: parent, create_statement: create_statement, extra_statements: extra_statements }.delete_if { |_, v| v.nil? } req = Google::Gax::to_proto(req, Google::Spanner::Admin::Database::V1::CreateDatabaseRequest) operation = Google::Gax::Operation.new( @create_database.call(req, ), @operations_client, Google::Spanner::Admin::Database::V1::Database, Google::Spanner::Admin::Database::V1::CreateDatabaseMetadata, call_options: ) operation.on_done { |operation| yield(operation) } if block_given? operation end |
#drop_database(database, options: nil) {|result, operation| ... } ⇒ Object
Drops (aka deletes) a Cloud Spanner database.
597 598 599 600 601 602 603 604 605 606 607 |
# File 'lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb', line 597 def drop_database \ database, options: nil, &block req = { database: database }.delete_if { |_, v| v.nil? } req = Google::Gax::to_proto(req, Google::Spanner::Admin::Database::V1::DropDatabaseRequest) @drop_database.call(req, , &block) nil end |
#get_database(name, options: nil) {|result, operation| ... } ⇒ Google::Spanner::Admin::Database::V1::Database
Gets the state of a Cloud Spanner database.
470 471 472 473 474 475 476 477 478 479 |
# File 'lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb', line 470 def get_database \ name, options: nil, &block req = { name: name }.delete_if { |_, v| v.nil? } req = Google::Gax::to_proto(req, Google::Spanner::Admin::Database::V1::GetDatabaseRequest) @get_database.call(req, , &block) end |
#get_database_ddl(database, options: nil) {|result, operation| ... } ⇒ Google::Spanner::Admin::Database::V1::GetDatabaseDdlResponse
Returns the schema of a Cloud Spanner database as a list of formatted DDL statements. This method does not show pending schema updates, those may be queried using the Operations API.
630 631 632 633 634 635 636 637 638 639 |
# File 'lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb', line 630 def get_database_ddl \ database, options: nil, &block req = { database: database }.delete_if { |_, v| v.nil? } req = Google::Gax::to_proto(req, Google::Spanner::Admin::Database::V1::GetDatabaseDdlRequest) @get_database_ddl.call(req, , &block) end |
#get_iam_policy(resource, options: nil) {|result, operation| ... } ⇒ Google::Iam::V1::Policy
Gets the access control policy for a database resource. Returns an empty policy if a database exists but does not have a policy set.
Authorization requires spanner.databases.getIamPolicy permission on
resource.
712 713 714 715 716 717 718 719 720 721 |
# File 'lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb', line 712 def get_iam_policy \ resource, options: nil, &block req = { resource: resource }.delete_if { |_, v| v.nil? } req = Google::Gax::to_proto(req, Google::Iam::V1::GetIamPolicyRequest) @get_iam_policy.call(req, , &block) end |
#list_databases(parent, page_size: nil, options: nil) {|result, operation| ... } ⇒ Google::Gax::PagedEnumerable<Google::Spanner::Admin::Database::V1::Database>
Lists Cloud Spanner databases.
351 352 353 354 355 356 357 358 359 360 361 362 |
# File 'lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb', line 351 def list_databases \ parent, page_size: nil, options: nil, &block req = { parent: parent, page_size: page_size }.delete_if { |_, v| v.nil? } req = Google::Gax::to_proto(req, Google::Spanner::Admin::Database::V1::ListDatabasesRequest) @list_databases.call(req, , &block) end |
#set_iam_policy(resource, policy, options: nil) {|result, operation| ... } ⇒ Google::Iam::V1::Policy
Sets the access control policy on a database resource. Replaces any existing policy.
Authorization requires spanner.databases.setIamPolicy permission on
resource.
675 676 677 678 679 680 681 682 683 684 685 686 |
# File 'lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb', line 675 def set_iam_policy \ resource, policy, options: nil, &block req = { resource: resource, policy: policy }.delete_if { |_, v| v.nil? } req = Google::Gax::to_proto(req, Google::Iam::V1::SetIamPolicyRequest) @set_iam_policy.call(req, , &block) end |
#test_iam_permissions(resource, permissions, options: nil) {|result, operation| ... } ⇒ Google::Iam::V1::TestIamPermissionsResponse
Returns permissions that the caller has on the specified database resource.
Attempting this RPC on a non-existent Cloud Spanner database will result in
a NOT_FOUND error if the user has spanner.databases.list permission on
the containing Cloud Spanner instance. Otherwise returns an empty set of
permissions.
756 757 758 759 760 761 762 763 764 765 766 767 |
# File 'lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb', line 756 def \ resource, , options: nil, &block req = { resource: resource, permissions: }.delete_if { |_, v| v.nil? } req = Google::Gax::to_proto(req, Google::Iam::V1::TestIamPermissionsRequest) @test_iam_permissions.call(req, , &block) end |
#update_database_ddl(database, statements, operation_id: nil, options: nil) ⇒ Google::Gax::Operation
Updates the schema of a Cloud Spanner database by
creating/altering/dropping tables, columns, indexes, etc. The returned
long-running operation will have a name of
the format <database_name>/operations/<operation_id> and can be used to
track execution of the schema change(s). The
metadata field type is
UpdateDatabaseDdlMetadata.
The operation has no response.
557 558 559 560 561 562 563 564 565 566 567 568 569 570 571 572 573 574 575 576 577 |
# File 'lib/google/cloud/spanner/admin/database/v1/database_admin_client.rb', line 557 def update_database_ddl \ database, statements, operation_id: nil, options: nil req = { database: database, statements: statements, operation_id: operation_id }.delete_if { |_, v| v.nil? } req = Google::Gax::to_proto(req, Google::Spanner::Admin::Database::V1::UpdateDatabaseDdlRequest) operation = Google::Gax::Operation.new( @update_database_ddl.call(req, ), @operations_client, Google::Protobuf::Empty, Google::Spanner::Admin::Database::V1::UpdateDatabaseDdlMetadata, call_options: ) operation.on_done { |operation| yield(operation) } if block_given? operation end |