Class: Google::Cloud::Spanner::V1::SpannerClient

Inherits:
Object
  • Object
show all
Defined in:
lib/google/cloud/spanner/v1/spanner_client.rb

Overview

Cloud Spanner API

The Cloud Spanner API can be used to manage sessions and execute transactions on data stored in Cloud Spanner databases.

Constant Summary collapse

SERVICE_ADDRESS =

The default address of the service.

"spanner.googleapis.com".freeze
DEFAULT_SERVICE_PORT =

The default port of the service.

443
GRPC_INTERCEPTORS =

The default set of gRPC interceptors.

[]
DEFAULT_TIMEOUT =
30
ALL_SCOPES =

The scopes needed to make gRPC calls to all of the methods defined in this service.

[
  "https://www.googleapis.com/auth/cloud-platform",
  "https://www.googleapis.com/auth/spanner.data"
].freeze

Class Method Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(credentials: nil, scopes: ALL_SCOPES, client_config: {}, timeout: DEFAULT_TIMEOUT, metadata: nil, exception_transformer: nil, lib_name: nil, lib_version: "") ⇒ SpannerClient

Returns a new instance of SpannerClient.

Parameters:

  • credentials (Google::Auth::Credentials, String, Hash, GRPC::Core::Channel, GRPC::Core::ChannelCredentials, Proc) (defaults to: nil) —

    Provides the means for authenticating requests made by the client. This parameter can be many types. A Google::Auth::Credentials uses a the properties of its represented keyfile for authenticating requests made by this client. A String will be treated as the path to the keyfile to be used for the construction of credentials for this client. A Hash will be treated as the contents of a keyfile to be used for the construction of credentials for this client. A GRPC::Core::Channel will be used to make calls through. A GRPC::Core::ChannelCredentials for the setting up the RPC client. The channel credentials should already be composed with a GRPC::Core::CallCredentials object. A Proc will be used as an updater_proc for the Grpc channel. The proc transforms the metadata for requests, generally, to give OAuth credentials.

  • scopes (Array<String>) (defaults to: ALL_SCOPES) —

    The OAuth scopes for this service. This parameter is ignored if an updater_proc is supplied.

  • client_config (Hash) (defaults to: {}) —

    A Hash for call options for each method. See Google::Gax#construct_settings for the structure of this data. Falls back to the default config if not specified or the specified config is missing data points.

  • timeout (Numeric) (defaults to: DEFAULT_TIMEOUT) —

    The default timeout, in seconds, for calls made through this client.

  • metadata (Hash) (defaults to: nil) —

    Default metadata to be sent with each request. This can be overridden on a per call basis.

  • exception_transformer (Proc) (defaults to: nil) —

    An optional proc that intercepts any exceptions raised during an API call to inject custom error handling.



144
145
146
147
148
149
150
151
152
153
154
155
156
157
158
159
160
161
162
163
164
165
166
167
168
169
170
171
172
173
174
175
176
177
178
179
180
181
182
183
184
185
186
187
188
189
190
191
192
193
194
195
196
197
198
199
200
201
202
203
204
205
206
207
208
209
210
211
212
213
214
215
216
217
218
219
220
221
222
223
224
225
226
227
228
229
230
231
232
233
234
235
236
237
238
239
240
241
242
243
244
245
246
247
248
249
250
251
252
253
254
255
256
257
258
259
260
261
262
263
264
265
266
267
268
269
270
271
272
273
274
275
276
277
278
279
280
281
282
283
284
285
286
287
288
289
290
291
292
293
294
295
296
297
298
299
300
301
302
303
304
305
306
307
308
309
310
311
312
313
314
315
316
317
318
319
320
321
322
323
324
325
326
327
328
329
330
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 144

def initialize \
    credentials: nil,
    scopes: ALL_SCOPES,
    client_config: {},
    timeout: DEFAULT_TIMEOUT,
    metadata: nil,
    exception_transformer: nil,
    lib_name: nil,
    lib_version: ""
  # These require statements are intentionally placed here to initialize
  # the gRPC module only when it's required.
  # See https://github.com/googleapis/toolkit/issues/446
  require "google/gax/grpc"
  require "google/spanner/v1/spanner_services_pb"

  credentials ||= Google::Cloud::Spanner::V1::Credentials.default

  if credentials.is_a?(String) || credentials.is_a?(Hash)
    updater_proc = Google::Cloud::Spanner::V1::Credentials.new(credentials).updater_proc
  end
  if credentials.is_a?(GRPC::Core::Channel)
    channel = credentials
  end
  if credentials.is_a?(GRPC::Core::ChannelCredentials)
    chan_creds = credentials
  end
  if credentials.is_a?(Proc)
    updater_proc = credentials
  end
  if credentials.is_a?(Google::Auth::Credentials)
    updater_proc = credentials.updater_proc
  end

  package_version = Google::Cloud::Spanner::VERSION

  google_api_client = "gl-ruby/#{RUBY_VERSION}"
  google_api_client << " #{lib_name}/#{lib_version}" if lib_name
  google_api_client << " gapic/#{package_version} gax/#{Google::Gax::VERSION}"
  google_api_client << " grpc/#{GRPC::VERSION}"
  google_api_client.freeze

  headers = { :"x-goog-api-client" => google_api_client }
  headers.merge!() unless .nil?
  client_config_file = Pathname.new(__dir__).join(
    "spanner_client_config.json"
  )
  defaults = client_config_file.open do |f|
    Google::Gax.construct_settings(
      "google.spanner.v1.Spanner",
      JSON.parse(f.read),
      client_config,
      Google::Gax::Grpc::STATUS_CODE_NAMES,
      timeout,
      page_descriptors: PAGE_DESCRIPTORS,
      errors: Google::Gax::Grpc::API_ERRORS,
      metadata: headers
    )
  end

  # Allow overriding the service path/port in subclasses.
  service_path = self.class::SERVICE_ADDRESS
  port = self.class::DEFAULT_SERVICE_PORT
  interceptors = self.class::GRPC_INTERCEPTORS
  @spanner_stub = Google::Gax::Grpc.create_stub(
    service_path,
    port,
    chan_creds: chan_creds,
    channel: channel,
    updater_proc: updater_proc,
    scopes: scopes,
    interceptors: interceptors,
    &Google::Spanner::V1::Spanner::Stub.method(:new)
  )

  @create_session = Google::Gax.create_api_call(
    @spanner_stub.method(:create_session),
    defaults["create_session"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'database' => request.database}
    end
  )
  @get_session = Google::Gax.create_api_call(
    @spanner_stub.method(:get_session),
    defaults["get_session"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'name' => request.name}
    end
  )
  @list_sessions = Google::Gax.create_api_call(
    @spanner_stub.method(:list_sessions),
    defaults["list_sessions"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'database' => request.database}
    end
  )
  @delete_session = Google::Gax.create_api_call(
    @spanner_stub.method(:delete_session),
    defaults["delete_session"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'name' => request.name}
    end
  )
  @execute_sql = Google::Gax.create_api_call(
    @spanner_stub.method(:execute_sql),
    defaults["execute_sql"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'session' => request.session}
    end
  )
  @execute_streaming_sql = Google::Gax.create_api_call(
    @spanner_stub.method(:execute_streaming_sql),
    defaults["execute_streaming_sql"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'session' => request.session}
    end
  )
  @execute_batch_dml = Google::Gax.create_api_call(
    @spanner_stub.method(:execute_batch_dml),
    defaults["execute_batch_dml"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'session' => request.session}
    end
  )
  @read = Google::Gax.create_api_call(
    @spanner_stub.method(:read),
    defaults["read"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'session' => request.session}
    end
  )
  @streaming_read = Google::Gax.create_api_call(
    @spanner_stub.method(:streaming_read),
    defaults["streaming_read"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'session' => request.session}
    end
  )
  @begin_transaction = Google::Gax.create_api_call(
    @spanner_stub.method(:begin_transaction),
    defaults["begin_transaction"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'session' => request.session}
    end
  )
  @commit = Google::Gax.create_api_call(
    @spanner_stub.method(:commit),
    defaults["commit"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'session' => request.session}
    end
  )
  @rollback = Google::Gax.create_api_call(
    @spanner_stub.method(:rollback),
    defaults["rollback"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'session' => request.session}
    end
  )
  @partition_query = Google::Gax.create_api_call(
    @spanner_stub.method(:partition_query),
    defaults["partition_query"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'session' => request.session}
    end
  )
  @partition_read = Google::Gax.create_api_call(
    @spanner_stub.method(:partition_read),
    defaults["partition_read"],
    exception_transformer: exception_transformer,
    params_extractor: proc do |request|
      {'session' => request.session}
    end
  )
end

Class Method Details

.database_path(project, instance, database) ⇒ String

Returns a fully-qualified database resource name string.

Parameters:

  • project (String)
  • instance (String)
  • database (String)

Returns:

  • (String)


92
93
94
95
96
97
98
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 92

def self.database_path project, instance, database
  DATABASE_PATH_TEMPLATE.render(
    :"project" => project,
    :"instance" => instance,
    :"database" => database
  )
end

.session_path(project, instance, database, session) ⇒ String

Returns a fully-qualified session resource name string.

Parameters:

  • project (String)
  • instance (String)
  • database (String)
  • session (String)

Returns:

  • (String)


106
107
108
109
110
111
112
113
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 106

def self.session_path project, instance, database, session
  SESSION_PATH_TEMPLATE.render(
    :"project" => project,
    :"instance" => instance,
    :"database" => database,
    :"session" => session
  )
end

Instance Method Details

#begin_transaction(session, options_, options: nil) {|result, operation| ... } ⇒ Google::Spanner::V1::Transaction

Begins a new transaction. This step can often be skipped: Read, ExecuteSql and Commit can begin a new transaction as a side-effect.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_session = Google::Cloud::Spanner::V1::SpannerClient.session_path("[PROJECT]", "[INSTANCE]", "[DATABASE]", "[SESSION]")

# TODO: Initialize `options_`:
options_ = {}
response = spanner_client.begin_transaction(formatted_session, options_)

Parameters:

  • session (String) —

    Required. The session in which the transaction runs.

  • options_ (Google::Spanner::V1::TransactionOptions | Hash) —

    Required. Options for the new transaction. A hash of the same form as Google::Spanner::V1::TransactionOptions can also be provided.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Yields:

  • (result, operation) —

    Access the result along with the RPC operation

Yield Parameters:

Returns:

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



1131
1132
1133
1134
1135
1136
1137
1138
1139
1140
1141
1142
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 1131

def begin_transaction \
    session,
    options_,
    options: nil,
    &block
  req = {
    session: session,
    options: options_
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::BeginTransactionRequest)
  @begin_transaction.call(req, options, &block)
end

#commit(session, mutations, transaction_id: nil, single_use_transaction: nil, options: nil) {|result, operation| ... } ⇒ Google::Spanner::V1::CommitResponse

Commits a transaction. The request includes the mutations to be applied to rows in the database.

Commit might return an ABORTED error. This can occur at any time; commonly, the cause is conflicts with concurrent transactions. However, it can also happen for a variety of other reasons. If Commit returns ABORTED, the caller should re-attempt the transaction from the beginning, re-using the same session.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_session = Google::Cloud::Spanner::V1::SpannerClient.session_path("[PROJECT]", "[INSTANCE]", "[DATABASE]", "[SESSION]")

# TODO: Initialize `mutations`:
mutations = []
response = spanner_client.commit(formatted_session, mutations)

Parameters:

  • session (String) —

    Required. The session in which the transaction to be committed is running.

  • mutations (Array<Google::Spanner::V1::Mutation | Hash>) —

    The mutations to be executed when this transaction commits. All mutations are applied atomically, in the order they appear in this list. A hash of the same form as Google::Spanner::V1::Mutation can also be provided.

  • transaction_id (String) (defaults to: nil) —

    Commit a previously-started transaction.

  • single_use_transaction (Google::Spanner::V1::TransactionOptions | Hash) (defaults to: nil) —

    Execute mutations in a temporary transaction. Note that unlike commit of a previously-started transaction, commit with a temporary transaction is non-idempotent. That is, if the CommitRequest is sent to Cloud Spanner more than once (for instance, due to retries in the application, or in the transport library), it is possible that the mutations are executed more than once. If this is undesirable, use BeginTransaction and Commit instead. A hash of the same form as Google::Spanner::V1::TransactionOptions can also be provided.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Yields:

  • (result, operation) —

    Access the result along with the RPC operation

Yield Parameters:

Returns:

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



1193
1194
1195
1196
1197
1198
1199
1200
1201
1202
1203
1204
1205
1206
1207
1208
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 1193

def commit \
    session,
    mutations,
    transaction_id: nil,
    single_use_transaction: nil,
    options: nil,
    &block
  req = {
    session: session,
    mutations: mutations,
    transaction_id: transaction_id,
    single_use_transaction: single_use_transaction
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::CommitRequest)
  @commit.call(req, options, &block)
end

#create_session(database, session: nil, options: nil) {|result, operation| ... } ⇒ Google::Spanner::V1::Session

Creates a new session. A session can be used to perform transactions that read and/or modify data in a Cloud Spanner database. Sessions are meant to be reused for many consecutive transactions.

Sessions can only execute one transaction at a time. To execute multiple concurrent read-write/write-only transactions, create multiple sessions. Note that standalone reads and queries use a transaction internally, and count toward the one transaction limit.

Cloud Spanner limits the number of sessions that can exist at any given time; thus, it is a good idea to delete idle and/or unneeded sessions. Aside from explicit deletes, Cloud Spanner can delete sessions for which no operations are sent for more than an hour. If a session is deleted, requests to it return NOT_FOUND.

Idle sessions can be kept alive by sending a trivial SQL query periodically, e.g., "SELECT 1".

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_database = Google::Cloud::Spanner::V1::SpannerClient.database_path("[PROJECT]", "[INSTANCE]", "[DATABASE]")
response = spanner_client.create_session(formatted_database)

Parameters:

  • database (String) —

    Required. The database in which the new session is created.

  • session (Google::Spanner::V1::Session | Hash) (defaults to: nil) —

    The session to create. A hash of the same form as Google::Spanner::V1::Session can also be provided.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Yields:

  • (result, operation) —

    Access the result along with the RPC operation

Yield Parameters:

Returns:

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



375
376
377
378
379
380
381
382
383
384
385
386
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 375

def create_session \
    database,
    session: nil,
    options: nil,
    &block
  req = {
    database: database,
    session: session
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::CreateSessionRequest)
  @create_session.call(req, options, &block)
end

#delete_session(name, options: nil) {|result, operation| ... } ⇒ Object

Ends a session, releasing server resources associated with it. This will asynchronously trigger cancellation of any operations that are running with this session.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_name = Google::Cloud::Spanner::V1::SpannerClient.session_path("[PROJECT]", "[INSTANCE]", "[DATABASE]", "[SESSION]")
spanner_client.delete_session(formatted_name)

Parameters:

  • name (String) —

    Required. The name of the session to delete.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Yields:

  • (result, operation) —

    Access the result along with the RPC operation

Yield Parameters:

  • result —

    []

  • operation (GRPC::ActiveCall::Operation)

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



507
508
509
510
511
512
513
514
515
516
517
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 507

def delete_session \
    name,
    options: nil,
    &block
  req = {
    name: name
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::DeleteSessionRequest)
  @delete_session.call(req, options, &block)
  nil
end

#execute_batch_dml(session, transaction, statements, seqno, options: nil) {|result, operation| ... } ⇒ Google::Spanner::V1::ExecuteBatchDmlResponse

Executes a batch of SQL DML statements. This method allows many statements to be run with lower latency than submitting them sequentially with ExecuteSql.

Statements are executed in order, sequentially. ExecuteBatchDmlResponse will contain a ResultSet for each DML statement that has successfully executed. If a statement fails, its error status will be returned as part of the ExecuteBatchDmlResponse. Execution will stop at the first failed statement; the remaining statements will not run.

ExecuteBatchDml is expected to return an OK status with a response even if there was an error while processing one of the DML statements. Clients must inspect response.status to determine if there were any errors while processing the request.

See more details in ExecuteBatchDmlRequest and ExecuteBatchDmlResponse.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_session = Google::Cloud::Spanner::V1::SpannerClient.session_path("[PROJECT]", "[INSTANCE]", "[DATABASE]", "[SESSION]")

# TODO: Initialize `transaction`:
transaction = {}

# TODO: Initialize `statements`:
statements = []

# TODO: Initialize `seqno`:
seqno = 0
response = spanner_client.execute_batch_dml(formatted_session, transaction, statements, seqno)

Parameters:

  • session (String) —

    Required. The session in which the DML statements should be performed.

  • transaction (Google::Spanner::V1::TransactionSelector | Hash) —

    The transaction to use. A ReadWrite transaction is required. Single-use transactions are not supported (to avoid replay). The caller must either supply an existing transaction ID or begin a new transaction. A hash of the same form as Google::Spanner::V1::TransactionSelector can also be provided.

  • statements (Array<Google::Spanner::V1::ExecuteBatchDmlRequest::Statement | Hash>) —

    The list of statements to execute in this batch. Statements are executed serially, such that the effects of statement i are visible to statement i+1. Each statement must be a DML statement. Execution will stop at the first failed statement; the remaining statements will not run.

    REQUIRES: statements_size() > 0. A hash of the same form as Google::Spanner::V1::ExecuteBatchDmlRequest::Statement can also be provided.

  • seqno (Integer) —

    A per-transaction sequence number used to identify this request. This is used in the same space as the seqno in ExecuteSqlRequest. See more details in ExecuteSqlRequest.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Yields:

  • (result, operation) —

    Access the result along with the RPC operation

Yield Parameters:

Returns:

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



853
854
855
856
857
858
859
860
861
862
863
864
865
866
867
868
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 853

def execute_batch_dml \
    session,
    transaction,
    statements,
    seqno,
    options: nil,
    &block
  req = {
    session: session,
    transaction: transaction,
    statements: statements,
    seqno: seqno
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::ExecuteBatchDmlRequest)
  @execute_batch_dml.call(req, options, &block)
end

#execute_sql(session, sql, transaction: nil, params: nil, param_types: nil, resume_token: nil, query_mode: nil, partition_token: nil, seqno: nil, options: nil) {|result, operation| ... } ⇒ Google::Spanner::V1::ResultSet

Executes an SQL statement, returning all results in a single reply. This method cannot be used to return a result set larger than 10 MiB; if the query yields more data than that, the query fails with a FAILED_PRECONDITION error.

Operations inside read-write transactions might return ABORTED. If this occurs, the application should restart the transaction from the beginning. See Transaction for more details.

Larger result sets can be fetched in streaming fashion by calling ExecuteStreamingSql instead.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_session = Google::Cloud::Spanner::V1::SpannerClient.session_path("[PROJECT]", "[INSTANCE]", "[DATABASE]", "[SESSION]")

# TODO: Initialize `sql`:
sql = ''
response = spanner_client.execute_sql(formatted_session, sql)

Parameters:

  • session (String) —

    Required. The session in which the SQL query should be performed.

  • sql (String) —

    Required. The SQL string.

  • transaction (Google::Spanner::V1::TransactionSelector | Hash) (defaults to: nil) —

    The transaction to use. If none is provided, the default is a temporary read-only transaction with strong concurrency.

    The transaction to use.

    For queries, if none is provided, the default is a temporary read-only transaction with strong concurrency.

    Standard DML statements require a ReadWrite transaction. Single-use transactions are not supported (to avoid replay). The caller must either supply an existing transaction ID or begin a new transaction.

    Partitioned DML requires an existing PartitionedDml transaction ID. A hash of the same form as Google::Spanner::V1::TransactionSelector can also be provided.

  • params (Google::Protobuf::Struct | Hash) (defaults to: nil) —

    The SQL string can contain parameter placeholders. A parameter placeholder consists of '@' followed by the parameter name. Parameter names consist of any combination of letters, numbers, and underscores.

    Parameters can appear anywhere that a literal value is expected. The same parameter name can be used more than once, for example: "WHERE id > @msg_id AND id < @msg_id + 100"

    It is an error to execute an SQL statement with unbound parameters.

    Parameter values are specified using params, which is a JSON object whose keys are parameter names, and whose values are the corresponding parameter values. A hash of the same form as Google::Protobuf::Struct can also be provided.

  • param_types (Hash{String => Google::Spanner::V1::Type | Hash}) (defaults to: nil) —

    It is not always possible for Cloud Spanner to infer the right SQL type from a JSON value. For example, values of type BYTES and values of type STRING both appear in params as JSON strings.

    In these cases, param_types can be used to specify the exact SQL type for some or all of the SQL statement parameters. See the definition of Type for more information about SQL types. A hash of the same form as Google::Spanner::V1::Type can also be provided.

  • resume_token (String) (defaults to: nil) —

    If this request is resuming a previously interrupted SQL statement execution, resume_token should be copied from the last PartialResultSet yielded before the interruption. Doing this enables the new SQL statement execution to resume where the last one left off. The rest of the request parameters must exactly match the request that yielded this token.

  • query_mode (Google::Spanner::V1::ExecuteSqlRequest::QueryMode) (defaults to: nil) —

    Used to control the amount of debugging information returned in ResultSetStats. If partition_token is set, query_mode can only be set to QueryMode::NORMAL.

  • partition_token (String) (defaults to: nil) —

    If present, results will be restricted to the specified partition previously created using PartitionQuery(). There must be an exact match for the values of fields common to this message and the PartitionQueryRequest message used to create this partition_token.

  • seqno (Integer) (defaults to: nil) —

    A per-transaction sequence number used to identify this request. This makes each request idempotent such that if the request is received multiple times, at most one will succeed.

    The sequence number must be monotonically increasing within the transaction. If a request arrives for the first time with an out-of-order sequence number, the transaction may be aborted. Replays of previously handled requests will yield the same response as the first execution.

    Required for DML statements. Ignored for queries.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Yields:

  • (result, operation) —

    Access the result along with the RPC operation

Yield Parameters:

Returns:

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



630
631
632
633
634
635
636
637
638
639
640
641
642
643
644
645
646
647
648
649
650
651
652
653
654
655
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 630

def execute_sql \
    session,
    sql,
    transaction: nil,
    params: nil,
    param_types: nil,
    resume_token: nil,
    query_mode: nil,
    partition_token: nil,
    seqno: nil,
    options: nil,
    &block
  req = {
    session: session,
    sql: sql,
    transaction: transaction,
    params: params,
    param_types: param_types,
    resume_token: resume_token,
    query_mode: query_mode,
    partition_token: partition_token,
    seqno: seqno
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::ExecuteSqlRequest)
  @execute_sql.call(req, options, &block)
end

#execute_streaming_sql(session, sql, transaction: nil, params: nil, param_types: nil, resume_token: nil, query_mode: nil, partition_token: nil, seqno: nil, options: nil) ⇒ Enumerable<Google::Spanner::V1::PartialResultSet>

Like ExecuteSql, except returns the result set as a stream. Unlike ExecuteSql, there is no limit on the size of the returned result set. However, no individual row in the result set can exceed 100 MiB, and no column value can exceed 10 MiB.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_session = Google::Cloud::Spanner::V1::SpannerClient.session_path("[PROJECT]", "[INSTANCE]", "[DATABASE]", "[SESSION]")

# TODO: Initialize `sql`:
sql = ''
spanner_client.execute_streaming_sql(formatted_session, sql).each do |element|
  # Process element.
end

Parameters:

  • session (String) —

    Required. The session in which the SQL query should be performed.

  • sql (String) —

    Required. The SQL string.

  • transaction (Google::Spanner::V1::TransactionSelector | Hash) (defaults to: nil) —

    The transaction to use. If none is provided, the default is a temporary read-only transaction with strong concurrency.

    The transaction to use.

    For queries, if none is provided, the default is a temporary read-only transaction with strong concurrency.

    Standard DML statements require a ReadWrite transaction. Single-use transactions are not supported (to avoid replay). The caller must either supply an existing transaction ID or begin a new transaction.

    Partitioned DML requires an existing PartitionedDml transaction ID. A hash of the same form as Google::Spanner::V1::TransactionSelector can also be provided.

  • params (Google::Protobuf::Struct | Hash) (defaults to: nil) —

    The SQL string can contain parameter placeholders. A parameter placeholder consists of '@' followed by the parameter name. Parameter names consist of any combination of letters, numbers, and underscores.

    Parameters can appear anywhere that a literal value is expected. The same parameter name can be used more than once, for example: "WHERE id > @msg_id AND id < @msg_id + 100"

    It is an error to execute an SQL statement with unbound parameters.

    Parameter values are specified using params, which is a JSON object whose keys are parameter names, and whose values are the corresponding parameter values. A hash of the same form as Google::Protobuf::Struct can also be provided.

  • param_types (Hash{String => Google::Spanner::V1::Type | Hash}) (defaults to: nil) —

    It is not always possible for Cloud Spanner to infer the right SQL type from a JSON value. For example, values of type BYTES and values of type STRING both appear in params as JSON strings.

    In these cases, param_types can be used to specify the exact SQL type for some or all of the SQL statement parameters. See the definition of Type for more information about SQL types. A hash of the same form as Google::Spanner::V1::Type can also be provided.

  • resume_token (String) (defaults to: nil) —

    If this request is resuming a previously interrupted SQL statement execution, resume_token should be copied from the last PartialResultSet yielded before the interruption. Doing this enables the new SQL statement execution to resume where the last one left off. The rest of the request parameters must exactly match the request that yielded this token.

  • query_mode (Google::Spanner::V1::ExecuteSqlRequest::QueryMode) (defaults to: nil) —

    Used to control the amount of debugging information returned in ResultSetStats. If partition_token is set, query_mode can only be set to QueryMode::NORMAL.

  • partition_token (String) (defaults to: nil) —

    If present, results will be restricted to the specified partition previously created using PartitionQuery(). There must be an exact match for the values of fields common to this message and the PartitionQueryRequest message used to create this partition_token.

  • seqno (Integer) (defaults to: nil) —

    A per-transaction sequence number used to identify this request. This makes each request idempotent such that if the request is received multiple times, at most one will succeed.

    The sequence number must be monotonically increasing within the transaction. If a request arrives for the first time with an out-of-order sequence number, the transaction may be aborted. Replays of previously handled requests will yield the same response as the first execution.

    Required for DML statements. Ignored for queries.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Returns:

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



761
762
763
764
765
766
767
768
769
770
771
772
773
774
775
776
777
778
779
780
781
782
783
784
785
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 761

def execute_streaming_sql \
    session,
    sql,
    transaction: nil,
    params: nil,
    param_types: nil,
    resume_token: nil,
    query_mode: nil,
    partition_token: nil,
    seqno: nil,
    options: nil
  req = {
    session: session,
    sql: sql,
    transaction: transaction,
    params: params,
    param_types: param_types,
    resume_token: resume_token,
    query_mode: query_mode,
    partition_token: partition_token,
    seqno: seqno
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::ExecuteSqlRequest)
  @execute_streaming_sql.call(req, options)
end

#get_session(name, options: nil) {|result, operation| ... } ⇒ Google::Spanner::V1::Session

Gets a session. Returns NOT_FOUND if the session does not exist. This is mainly useful for determining whether a session is still alive.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_name = Google::Cloud::Spanner::V1::SpannerClient.session_path("[PROJECT]", "[INSTANCE]", "[DATABASE]", "[SESSION]")
response = spanner_client.get_session(formatted_name)

Parameters:

  • name (String) —

    Required. The name of the session to retrieve.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Yields:

  • (result, operation) —

    Access the result along with the RPC operation

Yield Parameters:

Returns:

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



409
410
411
412
413
414
415
416
417
418
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 409

def get_session \
    name,
    options: nil,
    &block
  req = {
    name: name
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::GetSessionRequest)
  @get_session.call(req, options, &block)
end

#list_sessions(database, page_size: nil, filter: nil, options: nil) {|result, operation| ... } ⇒ Google::Gax::PagedEnumerable<Google::Spanner::V1::Session>

Lists all sessions in a given database.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_database = Google::Cloud::Spanner::V1::SpannerClient.database_path("[PROJECT]", "[INSTANCE]", "[DATABASE]")

# Iterate over all results.
spanner_client.list_sessions(formatted_database).each do |element|
  # Process element.
end

# Or iterate over results one page at a time.
spanner_client.list_sessions(formatted_database).each_page do |page|
  # Process each page at a time.
  page.each do |element|
    # Process element.
  end
end

Parameters:

  • database (String) —

    Required. The database in which to list sessions.

  • page_size (Integer) (defaults to: nil) —

    The maximum number of resources contained in the underlying API response. If page streaming is performed per-resource, this parameter does not affect the return value. If page streaming is performed per-page, this determines the maximum number of resources in a page.

  • filter (String) (defaults to: nil) —

    An expression for filtering the results of the request. Filter rules are case insensitive. The fields eligible for filtering are:

    • labels.key where key is the name of a label

    Some examples of using filters are:

    • labels.env:* --> The session has the label "env".
      • labels.env:dev --> The session has the label "env" and the value of the label contains the string "dev".
  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Yields:

  • (result, operation) —

    Access the result along with the RPC operation

Yield Parameters:

Returns:

  • (Google::Gax::PagedEnumerable<Google::Spanner::V1::Session>) —

    An enumerable of Google::Spanner::V1::Session instances. See Google::Gax::PagedEnumerable documentation for other operations such as per-page iteration or access to the response object.

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



472
473
474
475
476
477
478
479
480
481
482
483
484
485
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 472

def list_sessions \
    database,
    page_size: nil,
    filter: nil,
    options: nil,
    &block
  req = {
    database: database,
    page_size: page_size,
    filter: filter
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::ListSessionsRequest)
  @list_sessions.call(req, options, &block)
end

#partition_query(session, sql, transaction: nil, params: nil, param_types: nil, partition_options: nil, options: nil) {|result, operation| ... } ⇒ Google::Spanner::V1::PartitionResponse

Creates a set of partition tokens that can be used to execute a query operation in parallel. Each of the returned partition tokens can be used by ExecuteStreamingSql to specify a subset of the query result to read. The same session and read-only transaction must be used by the PartitionQueryRequest used to create the partition tokens and the ExecuteSqlRequests that use the partition tokens.

Partition tokens become invalid when the session used to create them is deleted, is idle for too long, begins a new transaction, or becomes too old. When any of these happen, it is not possible to resume the query, and the whole operation must be restarted from the beginning.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_session = Google::Cloud::Spanner::V1::SpannerClient.session_path("[PROJECT]", "[INSTANCE]", "[DATABASE]", "[SESSION]")

# TODO: Initialize `sql`:
sql = ''
response = spanner_client.partition_query(formatted_session, sql)

Parameters:

  • session (String) —

    Required. The session used to create the partitions.

  • sql (String) —

    The query request to generate partitions for. The request will fail if the query is not root partitionable. The query plan of a root partitionable query has a single distributed union operator. A distributed union operator conceptually divides one or more tables into multiple splits, remotely evaluates a subquery independently on each split, and then unions all results.

    This must not contain DML commands, such as INSERT, UPDATE, or DELETE. Use ExecuteStreamingSql with a PartitionedDml transaction for large, partition-friendly DML operations.

  • transaction (Google::Spanner::V1::TransactionSelector | Hash) (defaults to: nil) —

    Read only snapshot transactions are supported, read/write and single use transactions are not. A hash of the same form as Google::Spanner::V1::TransactionSelector can also be provided.

  • params (Google::Protobuf::Struct | Hash) (defaults to: nil) —

    The SQL query string can contain parameter placeholders. A parameter placeholder consists of '@' followed by the parameter name. Parameter names consist of any combination of letters, numbers, and underscores.

    Parameters can appear anywhere that a literal value is expected. The same parameter name can be used more than once, for example: "WHERE id > @msg_id AND id < @msg_id + 100"

    It is an error to execute an SQL query with unbound parameters.

    Parameter values are specified using params, which is a JSON object whose keys are parameter names, and whose values are the corresponding parameter values. A hash of the same form as Google::Protobuf::Struct can also be provided.

  • param_types (Hash{String => Google::Spanner::V1::Type | Hash}) (defaults to: nil) —

    It is not always possible for Cloud Spanner to infer the right SQL type from a JSON value. For example, values of type BYTES and values of type STRING both appear in params as JSON strings.

    In these cases, param_types can be used to specify the exact SQL type for some or all of the SQL query parameters. See the definition of Type for more information about SQL types. A hash of the same form as Google::Spanner::V1::Type can also be provided.

  • partition_options (Google::Spanner::V1::PartitionOptions | Hash) (defaults to: nil) —

    Additional options that affect how many partitions are created. A hash of the same form as Google::Spanner::V1::PartitionOptions can also be provided.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Yields:

  • (result, operation) —

    Access the result along with the RPC operation

Yield Parameters:

Returns:

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



1338
1339
1340
1341
1342
1343
1344
1345
1346
1347
1348
1349
1350
1351
1352
1353
1354
1355
1356
1357
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 1338

def partition_query \
    session,
    sql,
    transaction: nil,
    params: nil,
    param_types: nil,
    partition_options: nil,
    options: nil,
    &block
  req = {
    session: session,
    sql: sql,
    transaction: transaction,
    params: params,
    param_types: param_types,
    partition_options: partition_options
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::PartitionQueryRequest)
  @partition_query.call(req, options, &block)
end

#partition_read(session, table, key_set, transaction: nil, index: nil, columns: nil, partition_options: nil, options: nil) {|result, operation| ... } ⇒ Google::Spanner::V1::PartitionResponse

Creates a set of partition tokens that can be used to execute a read operation in parallel. Each of the returned partition tokens can be used by StreamingRead to specify a subset of the read result to read. The same session and read-only transaction must be used by the PartitionReadRequest used to create the partition tokens and the ReadRequests that use the partition tokens. There are no ordering guarantees on rows returned among the returned partition tokens, or even within each individual StreamingRead call issued with a partition_token.

Partition tokens become invalid when the session used to create them is deleted, is idle for too long, begins a new transaction, or becomes too old. When any of these happen, it is not possible to resume the read, and the whole operation must be restarted from the beginning.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_session = Google::Cloud::Spanner::V1::SpannerClient.session_path("[PROJECT]", "[INSTANCE]", "[DATABASE]", "[SESSION]")

# TODO: Initialize `table`:
table = ''

# TODO: Initialize `key_set`:
key_set = {}
response = spanner_client.partition_read(formatted_session, table, key_set)

Parameters:

  • session (String) —

    Required. The session used to create the partitions.

  • table (String) —

    Required. The name of the table in the database to be read.

  • key_set (Google::Spanner::V1::KeySet | Hash) —

    Required. key_set identifies the rows to be yielded. key_set names the primary keys of the rows in table to be yielded, unless index is present. If index is present, then key_set instead names index keys in index.

    It is not an error for the key_set to name rows that do not exist in the database. Read yields nothing for nonexistent rows. A hash of the same form as Google::Spanner::V1::KeySet can also be provided.

  • transaction (Google::Spanner::V1::TransactionSelector | Hash) (defaults to: nil) —

    Read only snapshot transactions are supported, read/write and single use transactions are not. A hash of the same form as Google::Spanner::V1::TransactionSelector can also be provided.

  • index (String) (defaults to: nil) —

    If non-empty, the name of an index on table. This index is used instead of the table primary key when interpreting key_set and sorting result rows. See key_set for further information.

  • columns (Array<String>) (defaults to: nil) —

    The columns of table to be returned for each row matching this request.

  • partition_options (Google::Spanner::V1::PartitionOptions | Hash) (defaults to: nil) —

    Additional options that affect how many partitions are created. A hash of the same form as Google::Spanner::V1::PartitionOptions can also be provided.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Yields:

  • (result, operation) —

    Access the result along with the RPC operation

Yield Parameters:

Returns:

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



1431
1432
1433
1434
1435
1436
1437
1438
1439
1440
1441
1442
1443
1444
1445
1446
1447
1448
1449
1450
1451
1452
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 1431

def partition_read \
    session,
    table,
    key_set,
    transaction: nil,
    index: nil,
    columns: nil,
    partition_options: nil,
    options: nil,
    &block
  req = {
    session: session,
    table: table,
    key_set: key_set,
    transaction: transaction,
    index: index,
    columns: columns,
    partition_options: partition_options
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::PartitionReadRequest)
  @partition_read.call(req, options, &block)
end

#read(session, table, columns, key_set, transaction: nil, index: nil, limit: nil, resume_token: nil, partition_token: nil, options: nil) {|result, operation| ... } ⇒ Google::Spanner::V1::ResultSet

Reads rows from the database using key lookups and scans, as a simple key/value style alternative to ExecuteSql. This method cannot be used to return a result set larger than 10 MiB; if the read matches more data than that, the read fails with a FAILED_PRECONDITION error.

Reads inside read-write transactions might return ABORTED. If this occurs, the application should restart the transaction from the beginning. See Transaction for more details.

Larger result sets can be yielded in streaming fashion by calling StreamingRead instead.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_session = Google::Cloud::Spanner::V1::SpannerClient.session_path("[PROJECT]", "[INSTANCE]", "[DATABASE]", "[SESSION]")

# TODO: Initialize `table`:
table = ''

# TODO: Initialize `columns`:
columns = []

# TODO: Initialize `key_set`:
key_set = {}
response = spanner_client.read(formatted_session, table, columns, key_set)

Parameters:

  • session (String) —

    Required. The session in which the read should be performed.

  • table (String) —

    Required. The name of the table in the database to be read.

  • columns (Array<String>) —

    The columns of table to be returned for each row matching this request.

  • key_set (Google::Spanner::V1::KeySet | Hash) —

    Required. key_set identifies the rows to be yielded. key_set names the primary keys of the rows in table to be yielded, unless index is present. If index is present, then key_set instead names index keys in index.

    If the partition_token field is empty, rows are yielded in table primary key order (if index is empty) or index key order (if index is non-empty). If the partition_token field is not empty, rows will be yielded in an unspecified order.

    It is not an error for the key_set to name rows that do not exist in the database. Read yields nothing for nonexistent rows. A hash of the same form as Google::Spanner::V1::KeySet can also be provided.

  • transaction (Google::Spanner::V1::TransactionSelector | Hash) (defaults to: nil) —

    The transaction to use. If none is provided, the default is a temporary read-only transaction with strong concurrency. A hash of the same form as Google::Spanner::V1::TransactionSelector can also be provided.

  • index (String) (defaults to: nil) —

    If non-empty, the name of an index on table. This index is used instead of the table primary key when interpreting key_set and sorting result rows. See key_set for further information.

  • limit (Integer) (defaults to: nil) —

    If greater than zero, only the first limit rows are yielded. If limit is zero, the default is no limit. A limit cannot be specified if partition_token is set.

  • resume_token (String) (defaults to: nil) —

    If this request is resuming a previously interrupted read, resume_token should be copied from the last PartialResultSet yielded before the interruption. Doing this enables the new read to resume where the last read left off. The rest of the request parameters must exactly match the request that yielded this token.

  • partition_token (String) (defaults to: nil) —

    If present, results will be restricted to the specified partition previously created using PartitionRead(). There must be an exact match for the values of fields common to this message and the PartitionReadRequest message used to create this partition_token.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Yields:

  • (result, operation) —

    Access the result along with the RPC operation

Yield Parameters:

Returns:

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



963
964
965
966
967
968
969
970
971
972
973
974
975
976
977
978
979
980
981
982
983
984
985
986
987
988
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 963

def read \
    session,
    table,
    columns,
    key_set,
    transaction: nil,
    index: nil,
    limit: nil,
    resume_token: nil,
    partition_token: nil,
    options: nil,
    &block
  req = {
    session: session,
    table: table,
    columns: columns,
    key_set: key_set,
    transaction: transaction,
    index: index,
    limit: limit,
    resume_token: resume_token,
    partition_token: partition_token
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::ReadRequest)
  @read.call(req, options, &block)
end

#rollback(session, transaction_id, options: nil) {|result, operation| ... } ⇒ Object

Rolls back a transaction, releasing any locks it holds. It is a good idea to call this for any transaction that includes one or more Read or ExecuteSql requests and ultimately decides not to commit.

Rollback returns OK if it successfully aborts the transaction, the transaction was already aborted, or the transaction is not found. Rollback never returns ABORTED.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_session = Google::Cloud::Spanner::V1::SpannerClient.session_path("[PROJECT]", "[INSTANCE]", "[DATABASE]", "[SESSION]")

# TODO: Initialize `transaction_id`:
transaction_id = ''
spanner_client.rollback(formatted_session, transaction_id)

Parameters:

  • session (String) —

    Required. The session in which the transaction to roll back is running.

  • transaction_id (String) —

    Required. The transaction to roll back.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Yields:

  • (result, operation) —

    Access the result along with the RPC operation

Yield Parameters:

  • result —

    []

  • operation (GRPC::ActiveCall::Operation)

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



1241
1242
1243
1244
1245
1246
1247
1248
1249
1250
1251
1252
1253
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 1241

def rollback \
    session,
    transaction_id,
    options: nil,
    &block
  req = {
    session: session,
    transaction_id: transaction_id
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::RollbackRequest)
  @rollback.call(req, options, &block)
  nil
end

#streaming_read(session, table, columns, key_set, transaction: nil, index: nil, limit: nil, resume_token: nil, partition_token: nil, options: nil) ⇒ Enumerable<Google::Spanner::V1::PartialResultSet>

Like Read, except returns the result set as a stream. Unlike Read, there is no limit on the size of the returned result set. However, no individual row in the result set can exceed 100 MiB, and no column value can exceed 10 MiB.

Examples:

require "google/cloud/spanner/v1"

spanner_client = Google::Cloud::Spanner::V1::SpannerClient.new
formatted_session = Google::Cloud::Spanner::V1::SpannerClient.session_path("[PROJECT]", "[INSTANCE]", "[DATABASE]", "[SESSION]")

# TODO: Initialize `table`:
table = ''

# TODO: Initialize `columns`:
columns = []

# TODO: Initialize `key_set`:
key_set = {}
spanner_client.streaming_read(formatted_session, table, columns, key_set).each do |element|
  # Process element.
end

Parameters:

  • session (String) —

    Required. The session in which the read should be performed.

  • table (String) —

    Required. The name of the table in the database to be read.

  • columns (Array<String>) —

    The columns of table to be returned for each row matching this request.

  • key_set (Google::Spanner::V1::KeySet | Hash) —

    Required. key_set identifies the rows to be yielded. key_set names the primary keys of the rows in table to be yielded, unless index is present. If index is present, then key_set instead names index keys in index.

    If the partition_token field is empty, rows are yielded in table primary key order (if index is empty) or index key order (if index is non-empty). If the partition_token field is not empty, rows will be yielded in an unspecified order.

    It is not an error for the key_set to name rows that do not exist in the database. Read yields nothing for nonexistent rows. A hash of the same form as Google::Spanner::V1::KeySet can also be provided.

  • transaction (Google::Spanner::V1::TransactionSelector | Hash) (defaults to: nil) —

    The transaction to use. If none is provided, the default is a temporary read-only transaction with strong concurrency. A hash of the same form as Google::Spanner::V1::TransactionSelector can also be provided.

  • index (String) (defaults to: nil) —

    If non-empty, the name of an index on table. This index is used instead of the table primary key when interpreting key_set and sorting result rows. See key_set for further information.

  • limit (Integer) (defaults to: nil) —

    If greater than zero, only the first limit rows are yielded. If limit is zero, the default is no limit. A limit cannot be specified if partition_token is set.

  • resume_token (String) (defaults to: nil) —

    If this request is resuming a previously interrupted read, resume_token should be copied from the last PartialResultSet yielded before the interruption. Doing this enables the new read to resume where the last read left off. The rest of the request parameters must exactly match the request that yielded this token.

  • partition_token (String) (defaults to: nil) —

    If present, results will be restricted to the specified partition previously created using PartitionRead(). There must be an exact match for the values of fields common to this message and the PartitionReadRequest message used to create this partition_token.

  • options (Google::Gax::CallOptions) (defaults to: nil) —

    Overrides the default settings for this call, e.g, timeout, retries, etc.

Returns:

Raises:

  • (Google::Gax::GaxError) —

    if the RPC is aborted.



1075
1076
1077
1078
1079
1080
1081
1082
1083
1084
1085
1086
1087
1088
1089
1090
1091
1092
1093
1094
1095
1096
1097
1098
1099
# File 'lib/google/cloud/spanner/v1/spanner_client.rb', line 1075

def streaming_read \
    session,
    table,
    columns,
    key_set,
    transaction: nil,
    index: nil,
    limit: nil,
    resume_token: nil,
    partition_token: nil,
    options: nil
  req = {
    session: session,
    table: table,
    columns: columns,
    key_set: key_set,
    transaction: transaction,
    index: index,
    limit: limit,
    resume_token: resume_token,
    partition_token: partition_token
  }.delete_if { |_, v| v.nil? }
  req = Google::Gax::to_proto(req, Google::Spanner::V1::ReadRequest)
  @streaming_read.call(req, options)
end