Class: Google::Iam::V1::GetIamPolicyRequest

Inherits:
Object
  • Object
show all
Defined in:
lib/google/cloud/spanner/admin/database/v1/doc/google/iam/v1/iam_policy.rb,
lib/google/cloud/spanner/admin/instance/v1/doc/google/iam/v1/iam_policy.rb

Overview

Request message for GetIamPolicy method.

Instance Attribute Summary collapse

Instance Attribute Details

#resource ⇒ String

Returns REQUIRED: The resource for which the policy is being requested. See the operation documentation for the appropriate value for this field.

Returns:

  • (String) —

    REQUIRED: The resource for which the policy is being requested. See the operation documentation for the appropriate value for this field.



37
# File 'lib/google/cloud/spanner/admin/database/v1/doc/google/iam/v1/iam_policy.rb', line 37

class GetIamPolicyRequest; end