Class: Google::Spanner::Admin::Instance::V1::ListInstancesRequest

Inherits:
Object
  • Object
show all
Defined in:
lib/google/cloud/spanner/admin/instance/v1/doc/google/spanner/admin/instance/v1/spanner_instance_admin.rb

Overview

The request for ListInstances.

Instance Attribute Summary collapse

Instance Attribute Details

#filter ⇒ String

Returns An expression for filtering the results of the request. Filter rules are case insensitive. The fields eligible for filtering are:

  • name

    • display_name
    • labels.key where key is the name of a label

    Some examples of using filters are:

  • name:* --> The instance has a name.

  • name:Howl --> The instance's name contains the string "howl".

  • name:HOWL --> Equivalent to above.

  • NAME:howl --> Equivalent to above.

  • labels.env:* --> The instance has the label "env".

  • labels.env:dev --> The instance has the label "env" and the value of the label contains the string "dev".

  • name:howl labels.env:dev --> The instance's name contains "howl" and it has the label "env" with its value containing "dev".

Returns:

  • (String) —

    An expression for filtering the results of the request. Filter rules are case insensitive. The fields eligible for filtering are:

    • name

      • display_name
      • labels.key where key is the name of a label

      Some examples of using filters are:

    • name:* --> The instance has a name.

    • name:Howl --> The instance's name contains the string "howl".

    • name:HOWL --> Equivalent to above.

    • NAME:howl --> Equivalent to above.

    • labels.env:* --> The instance has the label "env".

    • labels.env:dev --> The instance has the label "env" and the value of the label contains the string "dev".

    • name:howl labels.env:dev --> The instance's name contains "howl" and it has the label "env" with its value containing "dev".



208
# File 'lib/google/cloud/spanner/admin/instance/v1/doc/google/spanner/admin/instance/v1/spanner_instance_admin.rb', line 208

class ListInstancesRequest; end

#page_size ⇒ Integer

Returns Number of instances to be returned in the response. If 0 or less, defaults to the server's maximum allowed page size.

Returns:

  • (Integer) —

    Number of instances to be returned in the response. If 0 or less, defaults to the server's maximum allowed page size.



208
# File 'lib/google/cloud/spanner/admin/instance/v1/doc/google/spanner/admin/instance/v1/spanner_instance_admin.rb', line 208

class ListInstancesRequest; end

#page_token ⇒ String

Returns If non-empty, page_token should contain a next_page_token from a previous ListInstancesResponse.

Returns:



208
# File 'lib/google/cloud/spanner/admin/instance/v1/doc/google/spanner/admin/instance/v1/spanner_instance_admin.rb', line 208

class ListInstancesRequest; end

#parent ⇒ String

Returns Required. The name of the project for which a list of instances is requested. Values are of the form projects/<project>.

Returns:

  • (String) —

    Required. The name of the project for which a list of instances is requested. Values are of the form projects/<project>.



208
# File 'lib/google/cloud/spanner/admin/instance/v1/doc/google/spanner/admin/instance/v1/spanner_instance_admin.rb', line 208

class ListInstancesRequest; end