Class: Google::Spanner::V1::Mutation::Write
- Inherits:
-
Object
- Object
- Google::Spanner::V1::Mutation::Write
- Defined in:
- lib/google/cloud/spanner/v1/doc/google/spanner/v1/mutation.rb
Overview
Arguments to insert, update, insert_or_update, and replace operations.
Instance Attribute Summary collapse
-
#columns ⇒ Array<String>
The names of the columns in table to be written.
-
#table ⇒ String
Required.
-
#values ⇒ Array<Google::Protobuf::ListValue>
The values to be written.
Instance Attribute Details
#columns ⇒ Array<String>
Returns The names of the columns in table to be written.
The list of columns must contain enough columns to allow Cloud Spanner to derive values for all primary key columns in the row(s) to be modified.
74 |
# File 'lib/google/cloud/spanner/v1/doc/google/spanner/v1/mutation.rb', line 74 class Write; end |
#table ⇒ String
Returns Required. The table whose rows will be written.
74 |
# File 'lib/google/cloud/spanner/v1/doc/google/spanner/v1/mutation.rb', line 74 class Write; end |
#values ⇒ Array<Google::Protobuf::ListValue>
Returns The values to be written. values can contain more than one
list of values. If it does, then multiple rows are written, one
for each entry in values. Each list in values must have
exactly as many entries as there are entries in
columns above. Sending
multiple lists is equivalent to sending multiple Mutations, each
containing one values entry and repeating
table and
columns. Individual values in
each list are encoded as described here.
74 |
# File 'lib/google/cloud/spanner/v1/doc/google/spanner/v1/mutation.rb', line 74 class Write; end |