Class: Google::Spanner::V1::ResultSetStats
- Inherits:
-
Object
- Object
- Google::Spanner::V1::ResultSetStats
- Defined in:
- lib/google/cloud/spanner/v1/doc/google/spanner/v1/result_set.rb
Overview
Additional statistics about a ResultSet or PartialResultSet.
Instance Attribute Summary collapse
-
#query_plan ⇒ Google::Spanner::V1::QueryPlan
QueryPlan for the query associated with this result.
-
#query_stats ⇒ Google::Protobuf::Struct
Aggregated statistics from the execution of the query.
-
#row_count_exact ⇒ Integer
Standard DML returns an exact count of rows that were modified.
-
#row_count_lower_bound ⇒ Integer
Partitioned DML does not offer exactly-once semantics, so it returns a lower bound of the rows modified.
Instance Attribute Details
#query_plan ⇒ Google::Spanner::V1::QueryPlan
191 |
# File 'lib/google/cloud/spanner/v1/doc/google/spanner/v1/result_set.rb', line 191 class ResultSetStats; end |
#query_stats ⇒ Google::Protobuf::Struct
191 |
# File 'lib/google/cloud/spanner/v1/doc/google/spanner/v1/result_set.rb', line 191 class ResultSetStats; end |
#row_count_exact ⇒ Integer
191 |
# File 'lib/google/cloud/spanner/v1/doc/google/spanner/v1/result_set.rb', line 191 class ResultSetStats; end |
#row_count_lower_bound ⇒ Integer
191 |
# File 'lib/google/cloud/spanner/v1/doc/google/spanner/v1/result_set.rb', line 191 class ResultSetStats; end |