Class: Google::Spanner::V1::RollbackRequest

Inherits:
Object
  • Object
show all
Defined in:
lib/google/cloud/spanner/v1/doc/google/spanner/v1/spanner.rb

Overview

The request for Rollback.

Instance Attribute Summary collapse

Instance Attribute Details

#session ⇒ String

Returns Required. The session in which the transaction to roll back is running.

Returns:

  • (String) —

    Required. The session in which the transaction to roll back is running.



274
# File 'lib/google/cloud/spanner/v1/doc/google/spanner/v1/spanner.rb', line 274

class RollbackRequest; end

#transaction_id ⇒ String

Returns Required. The transaction to roll back.

Returns:

  • (String) —

    Required. The transaction to roll back.



274
# File 'lib/google/cloud/spanner/v1/doc/google/spanner/v1/spanner.rb', line 274

class RollbackRequest; end