Method: Google::Cloud#spanner
- Defined in:
- lib/google-cloud-spanner.rb
#spanner(scope: nil, timeout: nil, client_config: nil) ⇒ Google::Cloud::Spanner::Project
Creates a new object for connecting to the Spanner service. Each call creates a new connection.
For more information on connecting to Google Cloud see the [Authentication Guide](googlecloudplatform.github.io/google-cloud-ruby/#/docs/guides/authentication).
61 62 63 64 65 |
# File 'lib/google-cloud-spanner.rb', line 61 def spanner scope: nil, timeout: nil, client_config: nil Google::Cloud.spanner @project, @keyfile, scope: scope, timeout: (timeout || @timeout), client_config: client_config end |